import {
  Loader, Metadata, Config, Http, IEventParam, IDictionary,
} from '@zeedhi/core';
import {
  Form, Button, Select, CheckboxMultiple, SpeedDial,
} from '@zeedhi/common';
import { PmeMask } from '@/config';
  
export class Pme00800CalculoPrevConsController {
  public reportController = Loader.getInstance('ReportController');

  public appController = Loader.getInstance('AppController');

  public utilities = Loader.getInstance('Utilities');

  public blockActivator = false;

  private impConfig = {
    IMPFILTRO: 'N',
    IMPSERV: 'N',
  };

  public defaultReport({ component }: IEventParam<Select>) {
    component.setValue(component.datasource.data[0]);
  }

  public setFilterTpCardapio({ component }: IEventParam<Select>) {
    const conditions = [
      {
        name: 'CDFILCENTPME',
        operator: '=',
        value: this.appController.getCdCentral(),
      },
      {
        name: 'NRPASTATCARD',
        operator: '=',
        value: null,
      },
    ];
    
    component.datasource.filter = conditions;
    component.datasource.get();
  }

  public setFilterAgreg({ component }: IEventParam<Select>) {
    const agrupUnid = Metadata.getInstance<Select>('GRUPFILI');
    const conditions = [
      {
        name: 'CDFILCENTPME',
        operator: '=',
        value: this.appController.getCdCentral(),
      },
      {
        name: 'CDGRUPFILI',
        operator: '=',
        value: agrupUnid.value ? agrupUnid.value : 'T',
      },
    ];
    
    component.datasource.filter = conditions;
    component.datasource.get();
  }

  public setFilterSubGrupProd({ component }: IEventParam<Select>) {
    const conditions = [
      {
        name: 'CDFILCENTPME',
        operator: '=',
        value: this.appController.getCdCentral(),
      },
    ];
    
    component.datasource.filter = conditions;
    component.datasource.get();
  }

  public generateReport({ component }: IEventParam<Button>) {
    const form = Metadata.getInstance<Form>('FORMCALCPREVCONS');
    if (this.utilities.centralIsValid() && this.formIsValid(form)) {
      const row: IDictionary = form.value;
      
      this.setParams(row);
      if (!row.CDFILCENTPME) this.utilities.showAlert('alertError', 'pme::centralNaoInformada', 'error');
      else {
        row.TAMACNIVEL = this.utilities.calcTamacNivel(row.NIVEL, 'produto');
        row.AGRUPNIVEL = row.NIVEL;
        if ((row.CD_REPORT_CODE === 'PME00801' || row.CD_REPORT_CODE === 'PME00803') && row.CDTIPOCUSTO === null) {
          this.utilities.showAlert('alertError', 'pme::tipoCustoNaoInformado', 'error');
        } else {
          this.reportController.callGenerateReport(component, row, 'report/calcPrevCons');
        }
      }
    }
  }
  
  private setParams(row: IDictionary) {
    const mask: string = (Config.masks.productMask as PmeMask).pattern;
    row.IMPFILTRO = this.impConfig.IMPFILTRO;
    row.IMPSERV = this.impConfig.IMPSERV;
    row.CDTIPOCUSTO = row.TIPOCUSTO ? row.TIPOCUSTO.CDTIPOCUSTO : null;
    row.NMTIPOCUSTO = row.TIPOCUSTO ? row.TIPOCUSTO.NMTIPOCUSTO : null;
    row.PRODINI = row.PRODINICIAL ? this.utilities.getValueWithMaskProduto(row.PRODINICIAL.CDARVPROD) : '';
    row.PRODFIN = row.PRODFINAL ? this.utilities.getValueWithMaskProduto(row.PRODFINAL.CDARVPROD) : '';
    row.CDFILCENTPME = this.appController.getCdCentral();
    row.NMFILCENTPME = this.appController.getNmCentral();
    row.REPORT_NAME = row.REPORT.REPORT_NAME;
    row.CD_REPORT_CODE = row.REPORT.CD_REPORT_CODE;
    row.PRODUCTMASK = this.utilities.formatReportMask(mask);
  }

  public calculateCons() {
    const form = Metadata.getInstance<Form>('FORMCALCPREVCONS');
    const elActions = Metadata.getInstance<SpeedDial>('actionsCalculoPrevConsumo');

    if (this.utilities.centralIsValid() && this.formIsValid(form)) {
      this.blockActivator = true;
      const row: IDictionary = form.value;
      row.CDFILCENTPME = this.appController.getCdCentral();
      if (!row.CDFILCENTPME) this.utilities.showAlert('alertError', 'pme::centralNaoInformada', 'error');
      else {
        this.utilities.activeLoadingRoundButton(elActions);
        Http.post('calculatePrevCons', { requestType: 'Row', row }).then(() => {
          this.utilities.showAlert('alertSuccess', 'pme::calculadoComSucesso', 'success');
        }).finally(() => {
          this.utilities.disableLoadingRoundButton(elActions);
          this.blockActivator = false;
        });
      }
    }
  }
  
  public activatorClick({ event }: IEventParam<Button>) {
    if (this.blockActivator) event?.stopPropagation();
  }

  private formIsValid(form: IDictionary) {
    let valid = false;
    const row: IDictionary = form.value;
    if (form.validate()) {
      const dtDe = row.DTDE;
      const dtAte = row.DTATE;

      if (this.utilities.dateIsValid(dtDe, dtAte)) {
        const prodIni = row.PRODINICIAL;
        const prodFin = row.PRODFINAL;
        if ((prodIni == null || prodFin == null)
            || this.utilities.prodIsValid(prodIni.CDARVPROD, prodFin.CDARVPROD)
        ) {
          valid = true;
        } else this.utilities.showAlert('alertError', 'pme::prodIniMaior', 'error');
      } else this.utilities.showAlert('alertError', 'pme::dataIniMaior', 'error');
    }
    return valid;
  }

  public setImpConfig({ component }: IEventParam<CheckboxMultiple>) {
    if (component.value) {
      const items = component.value.join('_');
      if (items.includes('IMPFILTRO')) this.impConfig.IMPFILTRO = 'S';
      else this.impConfig.IMPFILTRO = 'N';

      if (items.includes('IMPSERV')) this.impConfig.IMPSERV = 'S';
      else this.impConfig.IMPSERV = 'N';
    }
  }

  public clearFields({ component }: IEventParam<Select>) {
    const unidade = Metadata.getInstance<Select>('UNIDADE');
    const servico = Metadata.getInstance<Select>('SERVICO');

    if (component.name === 'TIPOCARD') {
      servico.setValue('');
      unidade.setValue('');
    } else unidade.setValue('');
  }
}
